@@ -7,22 +7,27 @@
 # autorevision - extracts metadata about the head version from your
 # repository.
 
+# shellcheck disable=SC2154
+
 # Usage message.
 arUsage() {
-	cat > "/dev/stderr" << EOF
-usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-V]
+	tee >&2 << EOF
+usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-e name] [-U] [-V]
 	Options include:
 	-t output-type		= specify output type
 	-s symbol		= specify symbol output
 	-o cache-file		= specify cache file location
 	-f			= force the use of cache data
+	-e name			= set a different output name for VCS_EXTRA
 	-U			= check for untracked files in svn
 	-V			= emit version and exit
 	-?			= help message
 
 The following are valid output types:
-	clojure			= clojure file
 	c			= C/C++ file
+	clojure			= clojure file
+	cmake 			= CMake script file
+	csharp			= CSharp properties file
 	h			= Header for use with c/c++
 	hpp			= Alternate C++ header strings with namespace
 	ini			= INI file
@@ -65,8 +70,7 @@ EOF
 
 # Config
 ARVERSION="&&ARVERSION&&"
-TARGETFILE="/dev/stdout"
-while getopts ":t:o:s:VfU" OPTION; do
+while getopts ":t:o:s:e:VfU" OPTION; do
 	case "${OPTION}" in
 		t)
 			AFILETYPE="${OPTARG}"
@@ -80,6 +84,9 @@ while getopts ":t:o:s:VfU" OPTION; do
 		s)
 			VAROUT="${OPTARG}"
 		;;
+		e)
+			EXTRA_NAME="${OPTARG}"
+		;;
 		U)
 			UNTRACKEDFILES="1"
 		;;
@@ -110,18 +117,30 @@ elif [ ! -f "${CACHEFILE}" ] && [ "${CACHEFORCE}" = "1" ]; then
 	exit 1
 fi
 
+# Only use the local keyword if it is there (Solaris we are looking at
+# you).
+if command -v local > /dev/null 2>&1; then
+	LOCAL="local"
+elif command -v typeset > /dev/null 2>&1; then
+	LOCAL="typeset"
+else
+	LOCAL=""
+fi
+
 # Make sure that the path we are given is one we can source
 # (dash, we are looking at you).
 if [ ! -z "${CACHEFILE}" ] && ! echo "${CACHEFILE}" | grep -q '^\.*/'; then
 	CACHEFILE="./${CACHEFILE}"
 fi
 
+GENERATED_HEADER="Generated by autorevision - do not hand-hack!"
+: "${EXTRA_NAME:="VCS_EXTRA"}"
 
 # Functions to extract data from different repo types.
 # For git repos
 # shellcheck disable=SC2039,SC2164,SC2155
 gitRepo() {
-	local oldPath="${PWD}"
+	${LOCAL} oldPath="${PWD}"
 
 	cd "$(git rev-parse --show-toplevel)"
 
@@ -129,9 +148,11 @@ gitRepo() {
 
 	VCS_BASENAME="$(basename "${PWD}")"
 
-	VCS_UUID="$(git rev-list --max-parents=0 --date-order --reverse HEAD 2>/dev/null | sed -n 1p)"
+	${LOCAL} currentRev="$(git rev-parse HEAD)"
+
+	VCS_UUID="$(git rev-list --max-parents=0 --date-order --reverse "${currentRev}" 2>/dev/null | sed -n 1p)"
 	if [ -z "${VCS_UUID}" ]; then
-		VCS_UUID="$(git rev-list --topo-order HEAD | tail -n 1)"
+		VCS_UUID="$(git rev-list --topo-order "${currentRev}" | tail -n 1)"
 	fi
 
 	# Is the working copy clean?
@@ -139,7 +160,7 @@ gitRepo() {
 	VCS_WC_MODIFIED="${?}"
 
 	# Enumeration of changesets
-	VCS_NUM="$(git rev-list --count HEAD 2>/dev/null)"
+	VCS_NUM="$(git rev-list --count "${currentRev}" 2>/dev/null)"
 	if [ -z "${VCS_NUM}" ]; then
 		echo "warning: Counting the number of revisions may be slower due to an outdated git version less than 1.7.2.3. If something breaks, please update it." 1>&2
 		VCS_NUM="$(git rev-list HEAD | wc -l)"
@@ -148,21 +169,21 @@ gitRepo() {
 	# This may be a git-svn remote.  If so, report the Subversion revision.
 	if [ -z "$(git config svn-remote.svn.url 2>/dev/null)" ]; then
 		# The full revision hash
-		VCS_FULL_HASH="$(git rev-parse HEAD)"
+		VCS_FULL_HASH="$(git rev-parse "${currentRev}")"
 
 		# The short hash
-		VCS_SHORT_HASH="$(echo "${VCS_FULL_HASH}" | cut -b 1-7)"
+		VCS_SHORT_HASH="$(git rev-parse --short "${currentRev}")"
 	else
 		# The git-svn revision number
-		VCS_FULL_HASH="$(git svn find-rev HEAD)"
+		VCS_FULL_HASH="$(git svn find-rev "${currentRev}")"
 		VCS_SHORT_HASH="${VCS_FULL_HASH}"
 	fi
 
 	# Current branch
-	VCS_BRANCH="$(git rev-parse --symbolic-full-name --verify "$(git name-rev --name-only --no-undefined HEAD 2>/dev/null)" 2>/dev/null | sed -e 's:refs/heads/::' | sed -e 's:refs/::')"
+	VCS_BRANCH="$(git rev-parse --symbolic-full-name --verify "$(git name-rev --name-only --no-undefined "${currentRev}" 2>/dev/null)" 2>/dev/null | sed -e 's:refs/heads/::' | sed -e 's:refs/::')"
 
 	# Cache the description
-	local DESCRIPTION="$(git describe --long --tags 2>/dev/null)"
+	${LOCAL} DESCRIPTION="$(git describe --long --tags "${currentRev}" 2>/dev/null)"
 
 	# Current or last tag ancestor (empty if no tags)
 	VCS_TAG="$(echo "${DESCRIPTION}" | sed -e "s:-g${VCS_SHORT_HASH}\$::" -e 's:-[0-9]*$::')"
@@ -175,16 +196,16 @@ gitRepo() {
 	fi
 
 	# Date of the current commit
-	VCS_DATE="$(TZ=UTC git show -s --date=iso-strict-local --pretty=format:%ad | sed -e 's|+00:00|Z|')"
+	VCS_DATE="$(TZ=UTC git show -s --date=iso-strict-local --pretty=format:%cd "${currentRev}" 2>/dev/null | sed -e 's|+00:00|Z|')"
 	if [ -z "${VCS_DATE}" ]; then
 		echo "warning: Action stamps require git version 2.7+." 1>&2
-		VCS_DATE="$(git log -1 --pretty=format:%ci | sed -e 's: :T:' -e 's: ::' -e 's|+00:00|Z|')"
-		local ASdis="1"
+		VCS_DATE="$(git log -1 --pretty=format:%ci "${currentRev}" | sed -e 's: :T:' -e 's: ::' -e 's|+00:00|Z|')"
+		${LOCAL} ASdis="1"
 	fi
 
 	# Action Stamp
 	if [ -z "${ASdis}" ]; then
-		VCS_ACTION_STAMP="${VCS_DATE}!$(git show -s --pretty=format:%cE)"
+		VCS_ACTION_STAMP="${VCS_DATE}!$(git show -s --pretty=format:%cE "${currentRev}")"
 	else
 		VCS_ACTION_STAMP=""
 	fi
@@ -195,7 +216,7 @@ gitRepo() {
 # For hg repos
 # shellcheck disable=SC2039,SC2164
 hgRepo() {
-	local oldPath="${PWD}"
+	${LOCAL} oldPath="${PWD}"
 
 	cd "$(hg root)"
 
@@ -248,7 +269,7 @@ hgRepo() {
 # For bzr repos
 # shellcheck disable=SC2039,SC2164
 bzrRepo() {
-	local oldPath="${PWD}"
+	${LOCAL} oldPath="${PWD}"
 
 	cd "$(bzr root)"
 
@@ -298,17 +319,17 @@ bzrRepo() {
 # For svn repos
 # shellcheck disable=SC2039,SC2164,SC2155
 svnRepo() {
-	local oldPath="${PWD}"
+	${LOCAL} oldPath="${PWD}"
 
 	VCS_TYPE="svn"
 
 	case "${PWD}" in
 	/*trunk*|/*branches*|/*tags*)
-		local fn="${PWD}"
+		${LOCAL} fn="${PWD}"
 		while [ "$(basename "${fn}")" != 'trunk' ] && [ "$(basename "${fn}")" != 'branches' ] && [ "$(basename "${fn}")" != 'tags' ] && [ "$(basename "${fn}")" != '/' ]; do
-			local fn="$(dirname "${fn}")"
+			${LOCAL} fn="$(dirname "${fn}")"
 		done
-		local fn="$(dirname "${fn}")"
+		${LOCAL} fn="$(dirname "${fn}")"
 		if [ "${fn}" = '/' ]; then
 			VCS_BASENAME="$(basename "${PWD}")"
 		else
@@ -321,7 +342,7 @@ svnRepo() {
 	VCS_UUID="$(svn info --xml | sed -n -e 's:<uuid>::' -e 's:</uuid>::p')"
 
 	# Cache svnversion output
-	local SVNVERSION="$(svnversion)"
+	${LOCAL} SVNVERSION="$(svnversion)"
 
 	# Is the working copy clean?
 	echo "${SVNVERSION}" | grep -q "M"
@@ -355,8 +376,8 @@ svnRepo() {
 	# Current branch
 	case "${PWD}" in
 	/*trunk*|/*branches*|/*tags*)
-		local lastbase=""
-		local fn="${PWD}"
+		${LOCAL} lastbase=""
+		${LOCAL} fn="${PWD}"
 		while :
 		do
 			base="$(basename "${fn}")"
@@ -370,8 +391,8 @@ svnRepo() {
 				VCS_BRANCH=""
 				break
 			fi
-			local lastbase="${base}"
-			local fn="$(dirname "${fn}")"
+			${LOCAL} lastbase="${base}"
+			${LOCAL} fn="$(dirname "${fn}")"
 		done
 		;;
 	*) VCS_BRANCH="" ;;
@@ -395,9 +416,10 @@ svnRepo() {
 
 # Functions to output data in different formats.
 # For bash output
+# First in list because it is used by autorevision
 shOutput() {
-	cat > "${TARGETFILE}" << EOF
-# Generated by autorevision - do not hand-hack!
+	tee << EOF
+# ${GENERATED_HEADER}
 
 VCS_TYPE="${VCS_TYPE}"
 VCS_BASENAME="${VCS_BASENAME}"
@@ -407,7 +429,7 @@ VCS_DATE="${VCS_DATE}"
 VCS_BRANCH="${VCS_BRANCH}"
 VCS_TAG="${VCS_TAG}"
 VCS_TICK="${VCS_TICK}"
-VCS_EXTRA="${VCS_EXTRA}"
+${EXTRA_NAME}="${VCS_EXTRA}"
 
 VCS_ACTION_STAMP="${VCS_ACTION_STAMP}"
 VCS_FULL_HASH="${VCS_FULL_HASH}"
@@ -421,8 +443,8 @@ EOF
 
 # For source C output
 cOutput() {
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/* ${GENERATED_HEADER} */
 
 const char *VCS_TYPE         = "${VCS_TYPE}";
 const char *VCS_BASENAME     = "${VCS_BASENAME}";
@@ -432,7 +454,7 @@ const char *VCS_DATE         = "${VCS_DATE}";
 const char *VCS_BRANCH       = "${VCS_BRANCH}";
 const char *VCS_TAG          = "${VCS_TAG}";
 const int VCS_TICK           = ${VCS_TICK};
-const char *VCS_EXTRA        = "${VCS_EXTRA}";
+const char *${EXTRA_NAME}        = "${VCS_EXTRA}";
 
 const char *VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
 const char *VCS_FULL_HASH    = "${VCS_FULL_HASH}";
@@ -444,10 +466,35 @@ const int VCS_WC_MODIFIED     = ${VCS_WC_MODIFIED};
 EOF
 }
 
+# For Cmake output
+cmakeOutput() {
+	tee << EOF
+# ${GENERATED_HEADER}
+
+set(VCS_TYPE ${VCS_TYPE})
+set(VCS_BASENAME ${VCS_BASENAME})
+set(VCS_UUID ${VCS_UUID})
+set(VCS_NUM ${VCS_NUM})
+set(VCS_DATE ${VCS_DATE})
+set(VCS_BRANCH ${VCS_BRANCH})
+set(VCS_TAG ${VCS_TAG})
+set(VCS_TICK ${VCS_TICK})
+set(${EXTRA_NAME} ${VCS_EXTRA})
+
+set(VCS_ACTION_STAMP ${VCS_ACTION_STAMP})
+set(VCS_FULL_HASH ${VCS_FULL_HASH})
+set(VCS_SHORT_HASH ${VCS_SHORT_HASH})
+
+set(VCS_WC_MODIFIED ${VCS_WC_MODIFIED})
+
+# end
+EOF
+}
+
 # For header output
 hOutput() {
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/* ${GENERATED_HEADER} */
 #ifndef AUTOREVISION_H
 #define AUTOREVISION_H
 
@@ -459,7 +506,7 @@ hOutput() {
 #define VCS_BRANCH		"${VCS_BRANCH}"
 #define VCS_TAG			"${VCS_TAG}"
 #define VCS_TICK		${VCS_TICK}
-#define VCS_EXTRA		"${VCS_EXTRA}"
+#define ${EXTRA_NAME}		"${VCS_EXTRA}"
 
 #define VCS_ACTION_STAMP	"${VCS_ACTION_STAMP}"
 #define VCS_FULL_HASH		"${VCS_FULL_HASH}"
@@ -475,8 +522,8 @@ EOF
 
 # A header output for use with xcode to populate info.plist strings
 xcodeOutput() {
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/* ${GENERATED_HEADER} */
 #ifndef AUTOREVISION_H
 #define AUTOREVISION_H
 
@@ -488,7 +535,7 @@ xcodeOutput() {
 #define VCS_BRANCH		${VCS_BRANCH}
 #define VCS_TAG			${VCS_TAG}
 #define VCS_TICK		${VCS_TICK}
-#define VCS_EXTRA		${VCS_EXTRA}
+#define ${EXTRA_NAME}		${VCS_EXTRA}
 
 #define VCS_ACTION_STAMP	${VCS_ACTION_STAMP}
 #define VCS_FULL_HASH		${VCS_FULL_HASH}
@@ -531,8 +578,8 @@ swiftOutput() {
 	else
 		VCS_ACTION_STAMP="\"${VCS_ACTION_STAMP}\""
 	fi
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/* ${GENERATED_HEADER} */
 
 let VCS_TYPE			= "${VCS_TYPE}"
 let VCS_BASENAME		= "${VCS_BASENAME}"
@@ -542,7 +589,7 @@ let VCS_DATE			= "${VCS_DATE}"
 let VCS_BRANCH:	String	= "${VCS_BRANCH}"
 let VCS_TAG:	String?	= ${VCS_TAG}
 let VCS_TICK:	Int?	= ${VCS_TICK}
-let VCS_EXTRA:	String?	= ${VCS_EXTRA}
+let ${EXTRA_NAME}:	String?	= ${VCS_EXTRA}
 
 let VCS_ACTION_STAMP:	String?	= ${VCS_ACTION_STAMP}
 let VCS_FULL_HASH:		String	= "${VCS_FULL_HASH}"
@@ -560,8 +607,8 @@ pyOutput() {
 		0) VCS_WC_MODIFIED="False" ;;
 		1) VCS_WC_MODIFIED="True" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-# Generated by autorevision - do not hand-hack!
+	tee << EOF
+# ${GENERATED_HEADER}
 
 VCS_TYPE = "${VCS_TYPE}"
 VCS_BASENAME = "${VCS_BASENAME}"
@@ -571,7 +618,7 @@ VCS_DATE = "${VCS_DATE}"
 VCS_BRANCH = "${VCS_BRANCH}"
 VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
-VCS_EXTRA = "${VCS_EXTRA}"
+${EXTRA_NAME} = "${VCS_EXTRA}"
 
 VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
@@ -585,26 +632,27 @@ EOF
 
 # For Perl output
 plOutput() {
-	cat << EOF
-# Generated by autorevision - do not hand-hack!
+	tee << EOF
+# ${GENERATED_HEADER}
 
-\$VCS_TYPE = "${VCS_TYPE}";
-\$VCS_BASENAME = "${VCS_BASENAME}"
-\$VCS_UUID = "${VCS_UUID}"
+\$VCS_TYPE = '${VCS_TYPE}';
+\$VCS_BASENAME = '${VCS_BASENAME}';
+\$VCS_UUID = '${VCS_UUID}';
 \$VCS_NUM = ${VCS_NUM};
-\$VCS_DATE = "${VCS_DATE}";
-\$VCS_BRANCH = "${VCS_BRANCH}";
-\$VCS_TAG = "${VCS_TAG}";
+\$VCS_DATE = '${VCS_DATE}';
+\$VCS_BRANCH = '${VCS_BRANCH}';
+\$VCS_TAG = '${VCS_TAG}';
 \$VCS_TICK = ${VCS_TICK};
-\$VCS_EXTRA = "${VCS_EXTRA}";
+\$${EXTRA_NAME} = '${VCS_EXTRA}';
 
-\$VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
-\$VCS_FULL_HASH = "${VCS_FULL_HASH}";
-\$VCS_SHORT_HASH = "${VCS_SHORT_HASH}";
+\$VCS_ACTION_STAMP = '${VCS_ACTION_STAMP}';
+\$VCS_FULL_HASH = '${VCS_FULL_HASH}';
+\$VCS_SHORT_HASH = '${VCS_SHORT_HASH}';
 
 \$VCS_WC_MODIFIED = ${VCS_WC_MODIFIED};
 
 # end
+1;
 EOF
 }
 
@@ -614,8 +662,8 @@ luaOutput() {
 		0) VCS_WC_MODIFIED="false" ;;
 		1) VCS_WC_MODIFIED="true" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
--- Generated by autorevision - do not hand-hack!
+	tee << EOF
+-- ${GENERATED_HEADER}
 
 VCS_TYPE = "${VCS_TYPE}"
 VCS_BASENAME = "${VCS_BASENAME}"
@@ -625,7 +673,7 @@ VCS_DATE = "${VCS_DATE}"
 VCS_BRANCH = "${VCS_BRANCH}"
 VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
-VCS_EXTRA = "${VCS_EXTRA}"
+${EXTRA_NAME} = "${VCS_EXTRA}"
 
 VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
@@ -643,9 +691,9 @@ phpOutput() {
 		0) VCS_WC_MODIFIED="false" ;;
 		1) VCS_WC_MODIFIED="true" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
+	tee << EOF
 <?php
-# Generated by autorevision - do not hand-hack!
+# ${GENERATED_HEADER}
 
 return array(
 	"VCS_TYPE" => "${VCS_TYPE}",
@@ -656,7 +704,7 @@ return array(
 	"VCS_BRANCH" => "${VCS_BRANCH}",
 	"VCS_TAG" => "${VCS_TAG}",
 	"VCS_TICK" => ${VCS_TICK},
-	"VCS_EXTRA" => "${VCS_EXTRA}",
+	"${EXTRA_NAME}" => "${VCS_EXTRA}",
 	"VCS_ACTION_STAMP" => "${VCS_ACTION_STAMP}",
 	"VCS_FULL_HASH" => "${VCS_FULL_HASH}",
 	"VCS_SHORT_HASH" => "${VCS_SHORT_HASH}",
@@ -674,8 +722,8 @@ iniOutput() {
 		0) VCS_WC_MODIFIED="false" ;;
 		1) VCS_WC_MODIFIED="true" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-; Generated by autorevision - do not hand-hack!
+	tee << EOF
+; ${GENERATED_HEADER}
 [VCS]
 VCS_TYPE = "${VCS_TYPE}"
 VCS_BASENAME = "${VCS_BASENAME}"
@@ -685,7 +733,7 @@ VCS_DATE = "${VCS_DATE}"
 VCS_BRANCH = "${VCS_BRANCH}"
 VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
-VCS_EXTRA = "${VCS_EXTRA}"
+${EXTRA_NAME} = "${VCS_EXTRA}"
 VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
 VCS_SHORT_HASH = "${VCS_SHORT_HASH}"
@@ -700,8 +748,8 @@ jsOutput() {
 		1) VCS_WC_MODIFIED="true" ;;
 		0) VCS_WC_MODIFIED="false" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-/** Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/** ${GENERATED_HEADER} */
 
 var autorevision = {
 	VCS_TYPE: "${VCS_TYPE}",
@@ -712,7 +760,7 @@ var autorevision = {
 	VCS_BRANCH: "${VCS_BRANCH}",
 	VCS_TAG: "${VCS_TAG}",
 	VCS_TICK: ${VCS_TICK},
-	VCS_EXTRA: "${VCS_EXTRA}",
+	${EXTRA_NAME}: "${VCS_EXTRA}",
 
 	VCS_ACTION_STAMP: "${VCS_ACTION_STAMP}",
 	VCS_FULL_HASH: "${VCS_FULL_HASH}",
@@ -736,8 +784,9 @@ jsonOutput() {
 		1) VCS_WC_MODIFIED="true" ;;
 		0) VCS_WC_MODIFIED="false" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
+	tee << EOF
 {
+	"_comment": "${GENERATED_HEADER}",
 	"VCS_TYPE": "${VCS_TYPE}",
 	"VCS_BASENAME": "${VCS_BASENAME}",
 	"VCS_UUID": "${VCS_UUID}",
@@ -746,7 +795,7 @@ jsonOutput() {
 	"VCS_BRANCH":"${VCS_BRANCH}",
 	"VCS_TAG": "${VCS_TAG}",
 	"VCS_TICK": ${VCS_TICK},
-	"VCS_EXTRA": "${VCS_EXTRA}",
+	"${EXTRA_NAME}": "${VCS_EXTRA}",
 
 	"VCS_ACTION_STAMP": "${VCS_ACTION_STAMP}",
 	"VCS_FULL_HASH": "${VCS_FULL_HASH}",
@@ -763,8 +812,8 @@ javaOutput() {
 		1) VCS_WC_MODIFIED="true" ;;
 		0) VCS_WC_MODIFIED="false" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	tee << EOF
+/* ${GENERATED_HEADER} */
 
 public class autorevision {
     public static final String VCS_TYPE = "${VCS_TYPE}";
@@ -775,7 +824,7 @@ public class autorevision {
     public static final String VCS_BRANCH = "${VCS_BRANCH}";
     public static final String VCS_TAG = "${VCS_TAG}";
     public static final long VCS_TICK = ${VCS_TICK};
-    public static final String VCS_EXTRA = "${VCS_EXTRA}";
+    public static final String ${EXTRA_NAME} = "${VCS_EXTRA}";
 
     public static final String VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
     public static final String VCS_FULL_HASH = "${VCS_FULL_HASH}";
@@ -786,14 +835,47 @@ public class autorevision {
 EOF
 }
 
+csharpOutput() {
+	case "${VCS_WC_MODIFIED}" in
+		1) VCS_WC_MODIFIED="true" ;;
+		0) VCS_WC_MODIFIED="false" ;;
+	esac
+	if [ "${EXTRA_NAME}" = "VCS_EXTRA" ]; then
+        EXTRA_NAME="VcsExtra"
+    fi
+    tee << EOF
+/* ${GENERATED_HEADER} */
+
+namespace AutoRevision
+{
+    public class VersionInfo
+    {
+        public static string VcsType = "${VCS_TYPE}";
+        public static string VcsBasename = "${VCS_BASENAME}";
+        public static string VcsUuid = "${VCS_UUID}";
+        public static string VcsNum = "${VCS_NUM}";
+        public static string VcsDate = "${VCS_DATE}";
+        public static string VcsBranch = "${VCS_DATE}";
+        public static string VcsTag = "${VCS_TAG}";
+        public static string VcsTick = "${VCS_TICK}";
+        public static string ${EXTRA_NAME} = "${VCS_EXTRA}";
+        public static string VcsActionStamp = "${VCS_ACTION_STAMP}";
+        public static string VcsFullHash = "${VCS_FULL_HASH}";
+        public static string VcsShortHash = "${VCS_SHORT_HASH}";
+        public static string VcsWcModified = "${VCS_WC_MODIFIED}";
+    }
+}
+EOF
+}
+
 # For Java properties output
 javapropOutput() {
 	case "${VCS_WC_MODIFIED}" in
 		1) VCS_WC_MODIFIED="true" ;;
 		0) VCS_WC_MODIFIED="false" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-# Generated by autorevision - do not hand-hack!
+	tee << EOF
+# ${GENERATED_HEADER}
 
 VCS_TYPE=${VCS_TYPE}
 VCS_BASENAME=${VCS_BASENAME}
@@ -803,7 +885,7 @@ VCS_DATE=${VCS_DATE}
 VCS_BRANCH=${VCS_BRANCH}
 VCS_TAG=${VCS_TAG}
 VCS_TICK=${VCS_TICK}
-VCS_EXTRA=${VCS_EXTRA}
+${EXTRA_NAME}=${VCS_EXTRA}
 
 VCS_ACTION_STAMP=${VCS_ACTION_STAMP}
 VCS_FULL_HASH=${VCS_FULL_HASH}
@@ -815,7 +897,8 @@ EOF
 
 # For m4 output
 m4Output() {
-	cat > "${TARGETFILE}" << EOF
+	tee << EOF
+dnl ${GENERATED_HEADER}
 define(\`VCS_TYPE', \`${VCS_TYPE}')dnl
 define(\`VCS_BASENAME', \`${VCS_BASENAME}')dnl
 define(\`VCS_UUID', \`${VCS_UUID}')dnl
@@ -824,7 +907,7 @@ define(\`VCS_DATE', \`${VCS_DATE}')dnl
 define(\`VCS_BRANCH', \`${VCS_BRANCH}')dnl
 define(\`VCS_TAG', \`${VCS_TAG}')dnl
 define(\`VCS_TICK', \`${VCS_TICK}')dnl
-define(\`VCS_EXTRA', \`${VCS_EXTRA}')dnl
+define(\`${EXTRA_NAME}', \`${VCS_EXTRA}')dnl
 define(\`VCS_ACTIONSTAMP', \`${VCS_ACTION_STAMP}')dnl
 define(\`VCS_FULLHASH', \`${VCS_FULL_HASH}')dnl
 define(\`VCS_SHORTHASH', \`${VCS_SHORT_HASH}')dnl
@@ -838,8 +921,11 @@ texOutput() {
 		0) VCS_WC_MODIFIED="false" ;;
 		1) VCS_WC_MODIFIED="true" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-% Generated by autorevision - do not hand-hack!
+	if [ "${EXTRA_NAME}" = "VCS_EXTRA" ]; then
+		EXTRA_NAME="vcsExtra"
+	fi
+	tee << EOF
+% ${GENERATED_HEADER}
 \def \vcsType {${VCS_TYPE}}
 \def \vcsBasename {${VCS_BASENAME}}
 \def \vcsUUID {${VCS_UUID}}
@@ -848,7 +934,7 @@ texOutput() {
 \def \vcsBranch {${VCS_BRANCH}}
 \def \vcsTag {${VCS_TAG}}
 \def \vcsTick {${VCS_TICK}}
-\def \vcsExtra {${VCS_EXTRA}}
+\def \\${EXTRA_NAME} {${VCS_EXTRA}}
 \def \vcsACTIONSTAMP {${VCS_ACTION_STAMP}}
 \def \vcsFullHash {${VCS_FULL_HASH}}
 \def \vcsShortHash {${VCS_SHORT_HASH}}
@@ -863,8 +949,8 @@ schemeOutput() {
 		0) VCS_WC_MODIFIED="#f" ;;
 		1) VCS_WC_MODIFIED="#t" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-;; Generated by autorevision - do not hand-hack!
+	tee << EOF
+;; ${GENERATED_HEADER}
 (define VCS_TYPE        "${VCS_TYPE}")
 (define VCS_BASENAME    "${VCS_BASENAME}")
 (define VCS_UUID        "${VCS_UUID}")
@@ -873,7 +959,7 @@ schemeOutput() {
 (define VCS_BRANCH      "${VCS_BRANCH}")
 (define VCS_TAG         "${VCS_TAG}")
 (define VCS_TICK        ${VCS_TICK})
-(define VCS_EXTRA       "${VCS_EXTRA}")
+(define ${EXTRA_NAME}       "${VCS_EXTRA}")
 
 (define VCS_ACTION_STAMP   "${VCS_ACTION_STAMP}")
 (define VCS_FULL_HASH   "${VCS_FULL_HASH}")
@@ -890,8 +976,8 @@ clojureOutput() {
 		0) VCS_WC_MODIFIED="false" ;;
 		1) VCS_WC_MODIFIED="true" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-;; Generated by autorevision - do not hand-hack!
+	tee << EOF
+;; ${GENERATED_HEADER}
 (def VCS_TYPE        "${VCS_TYPE}")
 (def VCS_BASENAME    "${VCS_BASENAME}")
 (def VCS_UUID        "${VCS_UUID}")
@@ -900,7 +986,7 @@ clojureOutput() {
 (def VCS_BRANCH      "${VCS_BRANCH}")
 (def VCS_TAG         "${VCS_TAG}")
 (def VCS_TICK        ${VCS_TICK})
-(def VCS_EXTRA       "${VCS_EXTRA}")
+(def ${EXTRA_NAME}       "${VCS_EXTRA}")
 
 (def VCS_ACTION_STAMP   "${VCS_ACTION_STAMP}")
 (def VCS_FULL_HASH      "${VCS_FULL_HASH}")
@@ -913,8 +999,8 @@ EOF
 
 # For rpm spec file output
 rpmOutput() {
-	cat > "${TARGETFILE}" << EOF
-# Generated by autorevision - do not hand-hack!
+	tee << EOF
+# ${GENERATED_HEADER}
 $([ "${VCS_TYPE}" ] && echo "%define vcs_type		${VCS_TYPE}")
 $([ "${VCS_BASENAME}" ] && echo "%define vcs_basename		${VCS_BASENAME}")
 $([ "${VCS_UUID}" ] && echo "%define vcs_uuid		${VCS_UUID}")
@@ -923,7 +1009,7 @@ $([ "${VCS_DATE}" ] && echo "%define vcs_date		${VCS_DATE}")
 $([ "${VCS_BRANCH}" ] && echo "%define vcs_branch		${VCS_BRANCH}")
 $([ "${VCS_TAG}" ] && echo "%define vcs_tag			${VCS_TAG}")
 $([ "${VCS_TICK}" ] && echo "%define vcs_tick		${VCS_TICK}")
-$([ "${VCS_EXTRA}" ] && echo "%define vcs_extra		${VCS_EXTRA}")
+$([ "${VCS_EXTRA}" ] && echo "%define ${EXTRA_NAME}		${VCS_EXTRA}")
 
 $([ "${VCS_ACTION_STAMP}" ] && echo "%define vcs_action_stamp		${VCS_ACTION_STAMP}")
 $([ "${VCS_FULL_HASH}" ] && echo "%define vcs_full_hash		${VCS_FULL_HASH}")
@@ -934,15 +1020,18 @@ $([ "${VCS_WC_MODIFIED}" ] && echo "%define vcs_wc_modified		${VCS_WC_MODIFIED}"
 EOF
 }
 
+# For C++ Header output
 # shellcheck disable=SC2155,SC2039
 hppOutput() {
-	local NAMESPACE="$(echo "${VCS_BASENAME}" | sed -e 's:_::g' | tr '[:lower:]' '[:upper:]')"
-	cat > "${TARGETFILE}" << EOF
-/* Generated by autorevision - do not hand-hack! */
+	${LOCAL} NAMESPACE="$(echo "${VCS_BASENAME}" | sed -e 's:_::g' | tr '[:lower:]' '[:upper:]')"
+	tee << EOF
+/* ${GENERATED_HEADER} */
 
 #ifndef ${NAMESPACE}_AUTOREVISION_H
 #define ${NAMESPACE}_AUTOREVISION_H
 
+#include <string>
+
 namespace $(echo "${NAMESPACE}" | tr '[:upper:]' '[:lower:]')
 {
 	const std::string VCS_TYPE		= "${VCS_TYPE}";
@@ -953,7 +1042,7 @@ namespace $(echo "${NAMESPACE}" | tr '[:upper:]' '[:lower:]')
 	const std::string VCS_BRANCH	= "${VCS_BRANCH}";
 	const std::string VCS_TAG		= "${VCS_TAG}";
 	const int VCS_TICK				= ${VCS_TICK};
-	const std::string VCS_EXTRA		= "${VCS_EXTRA}";
+	const std::string ${EXTRA_NAME}		= "${VCS_EXTRA}";
 
 	const std::string VCS_ACTION_STAMP	= "${VCS_ACTION_STAMP}";
 	const std::string VCS_FULL_HASH		= "${VCS_FULL_HASH}";
@@ -968,13 +1057,14 @@ namespace $(echo "${NAMESPACE}" | tr '[:upper:]' '[:lower:]')
 EOF
 }
 
+# For Matlab output
 matlabOutput() {
 	case "${VCS_WC_MODIFIED}" in
 		0) VCS_WC_MODIFIED="FALSE" ;;
 		1) VCS_WC_MODIFIED="TRUE" ;;
 	esac
-	cat > "${TARGETFILE}" << EOF
-% Generated by autorevision - do not hand-hack!
+	tee << EOF
+% ${GENERATED_HEADER}
 
 VCS_TYPE = '${VCS_TYPE}';
 VCS_BASENAME = '${VCS_BASENAME}';
@@ -984,7 +1074,7 @@ VCS_DATE = '${VCS_DATE}';
 VCS_BRANCH = '${VCS_BRANCH}';
 VCS_TAG = '${VCS_TAG}';
 VCS_TICK = ${VCS_TICK};
-VCS_EXTRA = '${VCS_EXTRA}';
+${EXTRA_NAME} = '${VCS_EXTRA}';
 
 VCS_ACTION_STAMP = '${VCS_ACTION_STAMP}';
 VCS_FULL_HASH = '${VCS_FULL_HASH}';
@@ -996,9 +1086,10 @@ VCS_WC_MODIFIED = ${VCS_WC_MODIFIED};
 EOF
 }
 
+# For Octave output
 octaveOutput() {
-	cat > "${TARGETFILE}" << EOF
-% Generated by autorevision - do not hand-hack!
+	tee << EOF
+% ${GENERATED_HEADER}
 
 VCS_TYPE = '${VCS_TYPE}';
 VCS_BASENAME = '${VCS_BASENAME}';
@@ -1008,7 +1099,7 @@ VCS_DATE = '${VCS_DATE}';
 VCS_BRANCH = '${VCS_BRANCH}';
 VCS_TAG = '${VCS_TAG}';
 VCS_TICK = ${VCS_TICK};
-VCS_EXTRA = '${VCS_EXTRA}';
+${EXTRA_NAME} = '${VCS_EXTRA}';
 
 VCS_ACTION_STAMP = '${VCS_ACTION_STAMP}';
 VCS_FULL_HASH = '${VCS_FULL_HASH}';
@@ -1025,8 +1116,8 @@ EOF
 # Count path segments
 # shellcheck disable=SC2039
 pathSegment() {
-	local pathz="${1}"
-	local depth="0"
+	${LOCAL} pathz="${1}"
+	${LOCAL} depth="0"
 
 	if [ ! -z "${pathz}" ]; then
 		# Continue until we are at / or there are no path separators left.
@@ -1041,10 +1132,10 @@ pathSegment() {
 # Largest of four numbers
 # shellcheck disable=SC2039
 multiCompare() {
-	local larger="${1}"
-	local numA="${2}"
-	local numB="${3}"
-	local numC="${4}"
+	${LOCAL} larger="${1}"
+	${LOCAL} numA="${2}"
+	${LOCAL} numB="${3}"
+	${LOCAL} numC="${4}"
 
 	[ "${numA}" -gt "${larger}" ] && larger="${numA}"
 	[ "${numB}" -gt "${larger}" ] && larger="${numB}"
@@ -1056,50 +1147,59 @@ multiCompare() {
 # shellcheck disable=SC2155,SC2039
 repoTest() {
 	REPONUM="0"
-	if [ ! -z "$(git rev-parse HEAD 2>/dev/null)" ]; then
-		local gitPath="$(git rev-parse --show-toplevel)"
-		local gitDepth="$(pathSegment "${gitPath}")"
-		REPONUM="$((REPONUM+1))"
+	if command -v git > /dev/null 2>&1; then
+		${LOCAL} gitPath="$(git rev-parse --show-toplevel 2>/dev/null)"
+		${LOCAL} gitDepth="$(pathSegment "${gitPath}")"
+		if [ ! -z "${gitPath}" ]; then
+			REPONUM="$((REPONUM+1))"
+		fi
 	else
-		local gitDepth="0"
+		${LOCAL} gitDepth="0"
 	fi
-	if [ ! -z "$(hg root 2>/dev/null)" ]; then
-		local hgPath="$(hg root 2>/dev/null)"
-		local hgDepth="$(pathSegment "${hgPath}")"
-		REPONUM="$((REPONUM+1))"
+	if command -v hg > /dev/null 2>&1; then
+		${LOCAL} hgPath="$(hg root 2>/dev/null)"
+		${LOCAL} hgDepth="$(pathSegment "${hgPath}")"
+		if [ ! -z "${hgPath}" ]; then
+			REPONUM="$((REPONUM+1))"
+		fi
 	else
-		local hgDepth="0"
+		${LOCAL} hgDepth="0"
 	fi
-	if [ ! -z "$(bzr root 2>/dev/null)" ]; then
-		local bzrPath="$(bzr root 2>/dev/null)"
-		local bzrDepth="$(pathSegment "${bzrPath}")"
-		REPONUM="$((REPONUM+1))"
+	if command -v bzr > /dev/null 2>&1; then
+		${LOCAL} bzrPath="$(bzr root 2>/dev/null)"
+		${LOCAL} bzrDepth="$(pathSegment "${bzrPath}")"
+		if [ ! -z "${bzrPath}" ]; then
+			REPONUM="$((REPONUM+1))"
+		fi
 	else
-		local bzrDepth="0"
+		${LOCAL} bzrDepth="0"
 	fi
-	if [ ! -z "$(svn info 2>/dev/null)" ]; then
-		local stringz="<wcroot-abspath>"
-		local stringx="</wcroot-abspath>"
-		local svnPath="$(svn info --xml | sed -n -e "s:${stringz}::" -e "s:${stringx}::p")"
+	if command -v svn > /dev/null 2>&1; then
+		${LOCAL} stringz="<wcroot-abspath>"
+		${LOCAL} stringx="</wcroot-abspath>"
+		${LOCAL} svnPath="$(svn info --xml 2>/dev/null | sed -n -e "s:${stringz}::" -e "s:${stringx}::p")"
 		# An old enough svn will not be able give us a path; default
 		# to 1 for that case.
-		if [ -z  "${svnPath}" ]; then
-			local svnDepth="1"
+		if [ ! -z "${svnPath}" ]; then
+			${LOCAL} svnDepth="$(pathSegment "${svnPath}")"
+			REPONUM="$((REPONUM+1))"
+		elif [ -z "${svnPath}" ] && [ -d ".svn" ]; then
+			${LOCAL} svnDepth="1"
+			REPONUM="$((REPONUM+1))"
 		else
-			local svnDepth="$(pathSegment "${svnPath}")"
+			${LOCAL} svnDepth="0"
 		fi
-		REPONUM="$((REPONUM+1))"
 	else
-		local svnDepth="0"
+		${LOCAL} svnDepth="0"
 	fi
 
 	# Do not do more work then we have to.
 	if [ "${REPONUM}" = "0" ]; then
-		return
+		return 0
 	fi
 
 	# Figure out which repo is the deepest and use it.
-	local wonRepo="$(multiCompare "${gitDepth}" "${hgDepth}" "${bzrDepth}" "${svnDepth}")"
+	${LOCAL} wonRepo="$(multiCompare "${gitDepth}" "${hgDepth}" "${bzrDepth}" "${svnDepth}")"
 	if [ "${wonRepo}" = "${gitDepth}" ]; then
 		gitRepo
 	elif [ "${wonRepo}" = "${hgDepth}" ]; then
@@ -1196,6 +1296,8 @@ if [ ! -z "${AFILETYPE}" ]; then
 		javaOutput
 	elif [ "${AFILETYPE}" = "javaprop" ]; then
 		javapropOutput
+	elif [ "${AFILETYPE}" = "csharp" ]; then
+		csharpOutput
 	elif [ "${AFILETYPE}" = "tex" ]; then
 		texOutput
 	elif [ "${AFILETYPE}" = "m4" ]; then
@@ -1212,6 +1314,8 @@ if [ ! -z "${AFILETYPE}" ]; then
 		matlabOutput
 	elif [ "${AFILETYPE}" = "octave" ]; then
 		octaveOutput
+	elif [ "${AFILETYPE}" = "cmake" ]; then
+		cmakeOutput
 	else
 		echo "error: Not a valid output type." 1>&2
 		exit 1
@@ -1221,8 +1325,8 @@ fi
 
 # If requested, make a cache file.
 if [ ! -z "${CACHEFILE}" ] && [ ! "${CACHEFORCE}" = "1" ]; then
-	TARGETFILE="${CACHEFILE}.tmp"
-	shOutput
+	EXTRA_NAME="VCS_EXTRA"
+	shOutput > "${CACHEFILE}.tmp"
 
 	# Check to see if there have been any actual changes.
 	if [ ! -f "${CACHEFILE}" ]; then
